<?php
/*
Plugin Name: Simple Gmail Login
Plugin URI: http://victorjonsson.se/
Description: With this plugin you can login to wp-admin using your GMail credentials
Version: 1.1.2
Author: Victor Jonsson
Author URI: http://victorjonsson.se/
License URI: http://www.gnu.org/licenses/gpl-2.0.html
*/



/**
 * Helper class for this plugin
 */
class SimpleGmail_Plugin
{

    /**
     * Path to log file
     * @var string
     */
    private $log_file;

    /**
     * Whether or not log can be written plugin directory
     * @var bool
     */
    private $plugin_is_writeable = false;

    /**
     * Initiation of the plugin. Sets up all the different filters
     * and actions used by the plugin
     */
    function init()
    {
        $plugin_dir = dirname(__FILE__);
        $this->plugin_is_writeable = is_writable( $plugin_dir );
        $this->log_file = $plugin_dir.'/gmail-auth.log';

        if( basename($_SERVER['SCRIPT_FILENAME']) == 'wp-login.php' ) {
            // Login action
            add_action('login_init', array($this, 'loginAction'));
        }
        elseif( is_admin() ) {

            // Activate/deactivate
            register_activation_hook(__FILE__, array($this, 'activate'));
            register_deactivation_hook(__FILE__, array($this, 'deactivate'));

            // Settings page
            add_action('admin_menu', array($this, 'adminMenu'));

            // Dashboard widget
            add_action('wp_dashboard_setup', array($this, 'dashboardWidgetSetup'));
        }
    }

    /**
     *
     */
    function dashboardWidgetSetup()
    {
        if( current_user_can('activate_plugins') ) {
            wp_add_dashboard_widget(
                'simple-gmail-login',
                'Simple Gmail Login',
                array($this, 'authLogToHTML')
            );
        }
    }

    /**
     * Adds an option page to wp-admin where gmail log can be seen
     */
    function adminMenu()
    {
        add_options_page(
            'Simple Gmail Login',
            'Simple Gmail Login',
            'manage_options',
            'simple-gmail-login',
            array($this, 'adminOptionsPage')
        );
    }

    /**
     * Options page in wp-admin where gamil log can be seen
     */
    function adminOptionsPage()
    {
        ?>
        <div class="wrap">
            <div id="icon-options-general" class="icon32"><br></div>
            <h2>Simple Gmail Login</h2>
            <div style="padding-top: 12px">
                <?php if( $this->plugin_is_writeable ): ?>
                    <?php $this->authLogToHTML(400) ?>
                <?php else: ?>
                    <p style="background: red; color: #FFF;">
                        This plugin directory has to be writeable if you want the error log to be
                        displayed here. The messages is currently being written to the error log.
                    </p>
                <?php endif; ?>
            </div>
        </div>
        <?php
    }

    /**
     * @param int $height
     */
    function authLogToHTML($height = 190)
    {
        $lines = explode(PHP_EOL, file_get_contents($this->log_file));
        $lines = implode(PHP_EOL, array_reverse($lines));
        if( !$height )
            $height = 190;
        ?>
            <p>
                <strong>Authentication log:</strong>
            </p>
            <div style="max-width: 690px; height: <?php echo $height ?>px; overflow: hidden; overflow-y: scroll; padding: 5px 10px; line-height: 180%; background: #F2F2F2; border:#EEE solid 1px; border-radius: 4px">
                <?php echo nl2br($lines); ?>
            </div>
        <?php
    }

    /**
     * @param $email
     * @param $password
     * @throws SimpleGmail_AuthException
     */
    private function gmailAuthenticate($email, $password)
    {
        // No empty args
        if(empty($email) || empty($password))
            throw new SimpleGmail_AuthException('Neither e-mail nor password can be empty');

        // check if we have curl or not
        if( !function_exists('curl_init') )
            throw new SimpleGmail_AuthException('curl needs to be installed in order to use this function');

        $curl = curl_init('https://mail.google.com/mail/feed/atom');
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_FOLLOWLOCATION, 1);
        curl_setopt($curl, CURLOPT_USERPWD, $email.':'.$password);
        curl_exec($curl);

        $error = curl_error($curl);
        if( $error !== '' ) {
            $err_num = curl_errno($curl);
            curl_close($curl);
            throw new SimpleGmail_AuthException($error, $err_num);
        }
        else {
            // failed authentication
            $status_code = (int)curl_getinfo($curl, CURLINFO_HTTP_CODE);
            curl_close($curl);

            if($status_code !== 200) {
                throw new SimpleGmail_AuthException('Google responded with status '.$status_code);
            }
        }
    }

    /**
     * Action invoked when user is about to login
     */
    function loginAction()
    {
        if( !empty($_POST['log']) && isset($_POST['pwd']) ) {

            $login_by = filter_var($_POST['log'], FILTER_VALIDATE_EMAIL) !== false ? 'email':'login';

            if ( $user = get_user_by($login_by, sanitize_user($_POST['log'])) ) {

                try {
                    // Authenticate against gmail
                    $this->gmailAuthenticate($user->user_email, $_POST['pwd']);

                    // Set client as authenticated
                    wp_set_auth_cookie($user->ID, !empty($_POST['rememberme']));
                    do_action('wp_login', $user->user_login, $user);

                    // Log successful auth
                    $this->log('User #'.$user->ID.' ('.$user->user_email.') authenticated via Gmail', false);

                    // Redirect
                    $redirect_to = !empty($_REQUEST['redirect_to']) ? $_REQUEST['redirect_to'] : admin_url();
                    wp_safe_redirect($redirect_to);
                    die;

                } catch(SimpleGmail_AuthException $e) {
                    $this->log('Failed login for user #'.$user->ID.' ('.$user->user_email.') Message: '.$e->getMessage());
                }
            }

            // Fallback on ordinary wp login
        }
    }

    /**
     * Activation
     */
    function activate()
    {
        $this->log('Plugin activated', false);
    }

    /**
     * Deactivation
     */
    function deactivate()
    {
        if( $this->plugin_is_writeable && file_exists($this->log_file) )
            @unlink($this->log_file);
    }

    /**
     * Write message to plugin log file or error log
     * @param $mess
     * @param bool $is_error
     */
    private function log($mess, $is_error=true) {
        if( $this->plugin_is_writeable ) {
            $date = new DateTime('now', new DateTimeZone(get_option('timezone_string')));
            $mess = '<span style="color:'.( $is_error ? 'red':'green').'"> '.$date->format('\[Y-m-d H:i:s\]').'  '.$mess.' </span>';
            file_put_contents($this->log_file, $mess.PHP_EOL, FILE_APPEND);
            // Down size log file
            if( filesize($this->log_file) > (1024*1024) ) {
                file_put_contents($this->log_file, file_get_contents($this->log_file, null, null, 102400));
            }
        }
        else {
            error_log($mess);
        }
    }
}

/**
 * Exception thrown when authentication against GMail fails
 */
class SimpleGmail_AuthException extends Exception {}

// Run plugin
$simple_gmail_plugin = new SimpleGmail_Plugin();
$simple_gmail_plugin->init();